/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "cmdimport.h"
#include "shellinstance.h"
#include "shellcmdstringarg.h"
#include "fileiosystem.h"

namespace shellp
{

CmdImport::CmdImport(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pFileNameArg = new ShellCmdStringArg("filename");
	m_pFileNameArg->setDescription("Name of the file from which commands should be imported.");
	addArgument(m_pFileNameArg);

	setDescription("Import command lines from a file.");
}

CmdImport::~CmdImport()
{
	delete m_pFileNameArg;
}

bool CmdImport::execute()
{
	ShellInstance *pInst = ShellInstance::getInstance();
	IOSystem *pIOSys = pInst->getIOSystem();
	std::string fileName = m_pFileNameArg->getValue();

	FileIOSystem fileReader;

	if (!fileReader.init(fileName))
	{
		setErrorString(fileReader.getErrorString());
		return false;
	}

	std::string line;

	std::vector<std::string> newImportLines;

	while (fileReader.readInputLine("", line))
		newImportLines.push_back(line);

	for (int i = newImportLines.size()-1 ; i >= 0 ; i--)
		pIOSys->insertExtraInputLine(newImportLines[i]);

	return true;
}

} // end namespace

