/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "iosystem.h"
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include "debugnew.h"

namespace shellp
{

void IOSystem::print(const char *format,...)
{
	char str[8192];
	
	va_list ap;

	va_start(ap,format);
	vsnprintf(str,8191,format,ap);
	va_end(ap);
	str[8191] = 0;
	
	writeOutputLine(str,true);
}

void IOSystem::printNoNewLine(const char *format,...)
{
	char str[8192];
	
	va_list ap;

	va_start(ap,format);
	vsnprintf(str,8191,format,ap);
	va_end(ap);
	str[8191] = 0;
	
	writeOutputLine(str,false);
}

void IOSystem::setPercentageFeedback(double pct)
{
	char str[1024];

	//snprintf
	sprintf(str, "%f/100", pct);
	writeOutputLine(str, false);
}

bool IOSystem::plot2D(int id, const std::string &title,
			    const std::string &xLabel, const std::string &yLabel,
			    const std::vector<double> &xPoints, const std::vector<double> &yPoints,
			    bool forceShow)
{
	setErrorString("This IOSystem implementation does not provide an interactive 2D plot command");
	return false;
}

bool IOSystem::plot3D(int id, const std::string &title, 
			  const std::string &xLabel, const std::string &yLabel,
			  const std::string &zLabel,
			  double xOff, double yOff, double xDist, double yDist,
			  double xMultiplier, double yMultiplier, double zMultiplier,
			  const std::vector<double> &data, int numX, int numY, bool forceShow)
{
	setErrorString("This IOSystem implementation does not provide an interactive 3D plot command");
	return false;
}

void IOSystem::splitLine(const std::string &line, std::vector<std::string> &args)
{
	char *str = new char[line.size()+1];
	char *str2 = new char[line.size()+1];
	int len;
	char *p,*p2,*endptr;
	std::vector<std::string> arguments;
	
	strcpy(str,line.c_str());
	len = strlen(str);
	
	endptr = str+len;
	// copy arguments entries
	arguments.clear();
	
	p = str;
	while (p < endptr)
	{
		while (*p != 0 && (*p == ' ' || *p == '\t'))
			p++;
		p2 = p;

		bool done = false;

		while (!done)
		{
			while (*p2 != 0 && *p2 != ' ' && *p2 != '\t' && *p2 != '"')
				p2++;
			if (*p2 == '"')
			{
				p2++;
				while (*p2 != 0 && *p2 != '"')
					p2++;
				if (*p2 != 0)
					p2++;
			}
			else
				done = true;
		}
		
		*p2 = 0;
	
		char *p3;
		
		p2 = p;
		p3 = str2;
		while (*p2 != 0)
		{
			if (*p2 != '"')
			{
				*p3 = *p2;
				p3++;
			}
			p2++;
		}
		*p3 = 0;
		
		if (str2[0] != 0)
			arguments.push_back(std::string(str2));
		p = p2+1;
	}
	delete [] str;
	delete [] str2;

	// check for comments

	args.clear();
	
	bool found = false;
	
	for (int i = 0 ; !found && i < arguments.size() ; i++)
	{
		const char *str;
		int pos = 0;
		bool done = false;
		
		str = arguments[i].c_str();

		while (!done)
		{
			if (str[pos] == '#') // found comment
			{
				done = true;
				found = true;
			}
			else if (str[pos] == 0)
				done = true;
			else
				pos++;
		}

		if (pos > 0)
		{
			std::string newstr = arguments[i].substr(0,pos);
			args.push_back(newstr);
		}
	}
}

bool IOSystem::readInputLine(const std::string &prefix, std::string &line)
{
	if (!m_extraInputLines.empty())
	{
		line = *(m_extraInputLines.begin());
		m_extraInputLines.pop_front();
		return true;
	}

	writeInputPrefix(prefix);
	return readInputLineInternal(line);
}

bool IOSystem::readInputLineInternal(std::string &line)
{
	setErrorString("Reading input line is not implemented");
	return false;
}

} // end namespace
