/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#ifndef SHELLP_SHELL3DPLOTWINDOW_H

#define SHELLP_SHELL3DPLOTWINDOW_H

#include <QFrame>

namespace Qwt3D
{
	class GridPlot;
}

namespace shellp
{

class Shell3DPlotRequest
{
public:
	Shell3DPlotRequest(int id, const std::string &title, const std::string &xLabel, const std::string &yLabel,
		const std::string &zLabel, double xOff, double yOff, double xDist, double yDist,
		double **pData, int numX, int numY, bool forceShow)
	{
		m_id = id;
		m_title = title;
		m_xLabel = xLabel;
		m_yLabel = yLabel;
		m_zLabel = zLabel;
		m_xOffset = xOff;
		m_yOffset = yOff;
		m_xDist = xDist;
		m_yDist = yDist;
		m_numX = numX;
		m_numY = numY;
		m_pData = pData;
		m_forceShow = forceShow;
	}

	~Shell3DPlotRequest()
	{
		for (int i = 0 ; i < m_numX ; i++)
			delete [] m_pData[i];
		delete [] m_pData;
	}

	int getID() const								{ return m_id; }
	const std::string &getTitle() const						{ return m_title; }
	const std::string &getXLabel() const						{ return m_xLabel; }
	const std::string &getYLabel() const						{ return m_yLabel; }
	const std::string &getZLabel() const						{ return m_zLabel; }
	double getXOffset() const							{ return m_xOffset; }
	double getYOffset() const							{ return m_yOffset; }
	double getXPixelDistance() const						{ return m_xDist; }
	double getYPixelDistance() const						{ return m_yDist; }
	double **getData() const							{ return m_pData; }
	int getWidth() const								{ return m_numX; }
	int getHeight() const								{ return m_numY; }
	bool getForceShow() const							{ return m_forceShow; }
private:
	int m_id;
	std::string m_title, m_xLabel, m_yLabel, m_zLabel;
	double m_xOffset, m_yOffset, m_xDist, m_yDist;
	double **m_pData;
	int m_numX, m_numY;
	bool m_forceShow;
};

class Shell3DPlotWindow : public QWidget
{
	Q_OBJECT
public:
	Shell3DPlotWindow(QWidget *pParent = 0);
	~Shell3DPlotWindow();

	void processRequest(Shell3DPlotRequest *pRequest);
public slots:
	void toggleLogLinear();
	void togglePixelsOrPhysical();
private:
	void plotLinear();
	void plotLog();

	Qwt3D::GridPlot *m_pGridPlot;
	Shell3DPlotRequest *m_pRequest;
	bool m_log;
	bool m_plotPixels; // pixels instead of physical dimensions
};

} // end namespace

#endif // SHELLP_SHELL3DPLOTWINDOW_H
