/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#ifndef SHELLP_SHELLCMDARG_H

#define SHELLP_SHELLCMDARG_H

#include "shellconfig.h"
#include <errut/errorbase.h>
#include <string>
#include <vector>

namespace shellp
{

class SHELLP_IMPORTEXPORT ShellCmdArg : public errut::ErrorBase
{
public:
	enum ArgType { String, Integer, Real, Bool, Choice };
protected:
	ShellCmdArg(ArgType t, const std::string &name)				{ m_argType = t; m_argName = name; m_gotDefaultVal = false; m_expand = true; }
	ShellCmdArg(ArgType t, const std::string &name, 
			       const std::string &defaultVal)			{ m_argType = t; m_argName = name; m_gotDefaultVal = true; m_defaultValue = defaultVal; m_expand = true; }
public:
	virtual ~ShellCmdArg()							{ }

	ArgType getArgType() const						{ return m_argType; }
	std::string getName() const						{ return m_argName; }
	std::string getDescription() const					{ return m_description; }
	std::string getDefault() const						{ return m_defaultValue; }
	bool hasDefault() const							{ return m_gotDefaultVal; }

	void setExpand(bool f)							{ m_expand = f; }
	bool shouldExpand() const						{ return m_expand; }

	void setDescription(const std::string &d)				{ m_description = d; }
	void setDefault(const std::string &v)					{ m_gotDefaultVal = true; m_defaultValue = v; }
	void clearDefault() 							{ m_gotDefaultVal = false; }

	virtual bool parse(const std::string &arg) = 0;
	static bool parseArgumentList(std::vector<ShellCmdArg *> &args, 
			              const std::vector<std::string> &argStrings,
				      std::string &errStr);

	static std::string getArgumentNames(const std::vector<ShellCmdArg *> &args);
	static const char *getValidPreprocessorNameCharacters()			{ return m_pValidChars; }
private:
	static bool expandArgument(const std::string &arg, std::string &expandedArg, std::string &errStr, int iteration);

	ArgType m_argType;
	std::string m_argName, m_defaultValue, m_description;
	bool m_gotDefaultVal, m_expand;

	static const char m_pValidChars[];
};

} // end namespace

#endif // SHELLP_SHELLCMDARG_H

