/*
    
  This file is a part of Shellp, a 'shell helper' library.
  
  Copyright (C) 2011-2012 Jori Liesenborgs
  
  Contact: jori.liesenborgs@gmail.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
  USA

*/

#include "shellcmdintarg.h"
#include "shellinstance.h"
#include <stdlib.h>

#include "debugnew.h"

namespace shellp
{

ShellCmdIntArg::ShellCmdIntArg(const std::string &name) : ShellCmdArg(Integer, name)
{
	m_val = 0;
	m_usemin = false;
	m_usemax = false;
}

ShellCmdIntArg::ShellCmdIntArg(const std::string &name, const std::string &defaultValue) : ShellCmdArg(Integer, name, defaultValue)
{
	m_val = 0;
	m_usemin = false;
	m_usemax = false;
}

ShellCmdIntArg::~ShellCmdIntArg()
{
}

bool ShellCmdIntArg::parse(const std::string &arg)
{
	long int v;
	const char *nptr = arg.c_str();
	char *endptr;
	ShellInstance *pInst = ShellInstance::getInstance();
	
	v = strtol(nptr,&endptr,10); // base 10
	
	if (*nptr != '\0')
	{
		if (*endptr != '\0')
		{
			double value;
			bool positive = true;

			if (!pInst->evaluateExpression(arg, value))
			{
				setErrorString(std::string("Not a simple integer and math parser cannot parse it: ") + pInst->getErrorString());
				return false;
			}
			
			if (value < 0)
			{
				positive = false;
				value = -value;
			}

			v = (long int)(value + 0.5); // make sure it's rounded

			if (!positive)
				v = -v;
		}
	}

	m_val = (int)v;

	if (m_usemin && m_val < m_minval)
	{
		setErrorString("Value is smaller than the minimum value");
		return false;
	}
	if (m_usemax && m_val > m_maxval)
	{
		setErrorString("Value is larger than the maximum value");
		return false;
	}
	
	return true;
}

} // end namespace
